package us.camin.api.events;
import java.util.*;
import org.json.*;
import java.util.logging.Logger;
import java.util.logging.Level;

public abstract class Event {
  public int _id;
  public static Logger log;
  public static Map<String, Class> s_types;

  static {
    s_types = new HashMap<String, Class>();
    log = Logger.getLogger("CaminusEvents");
  }

  public static void registerHandler(String name, Class cls) {
    log.info("Registering event type "+name);
    s_types.put(name, cls);
  }

  public static Event fromJSON(JSONObject obj) throws JSONException {
    if (!s_types.containsKey(obj.getString("type"))) {
      log.log(Level.SEVERE, "Ignoring unknown event type "+obj.getString("type"));
      return null;
    }
    Class prototype = s_types.get(obj.getString("type"));
    Event ret = null;
    try {
      ret = (Event)prototype.getMethod("fromJSON").invoke(null, obj.getJSONObject("properties"));
    } catch (Exception e) {
      log.log(Level.SEVERE, "Could not instantiate event", e);
      return null;
    }
    ret._id = obj.getInt("id");
    return ret;
  }

  public JSONObject toJSON() throws JSONException {
      JSONWriter s = new JSONStringer();
      s.object();
      s.key("type");
      s.value(jsonName());
      s.key("payload");
      s.object();
      s = toJSON(s);
      s.endObject();
      s.endObject();
      return new JSONObject(((JSONStringer)s).toString());
  }

  public abstract JSONWriter toJSON(JSONWriter writer) throws JSONException;

  public abstract String jsonName();
}
